#!/bin/bash

#
# Install and configure pm2
#

function valid()
{
  if [ $1 -ne 0 ]; then
    printf "$2\n"
    exit 1
  fi
}

# Check if pm2 is already installed
npm list pm2 -g
if [ $? != 0 ]; then
  printf "Configuring pm2...\n"
  # Install pm2
  npm install pm2 -g
  valid $? "Error installing pm2."
  pm2 install pm2-logrotate
  valid $? "Error installing pm2-logrotate."
  # Allows the service to start with the system
  env PATH=$PATH:/usr/local/bin pm2 startup
  valid $? "Error adding pm2 to startup."
fi

# Add webapp to pm2 startup
(
  cd /opt/terrama2/4.0.8/webapp
  pm2 delete webapp

  pm2 start npm --name webapp -- start
  valid $? "Error executing webapp."
  pm2 save
  valid $? "Error saving pm2 state."
)

# Add webmonitor to pm2 startup
(
  cd /opt/terrama2/4.0.8/webmonitor
  pm2 delete webmonitor

  pm2 start npm --name webmonitor -- start
  valid $? "Error executing webmonitor."
  pm2 save
  valid $? "Error saving pm2 state."
)
